﻿using Stimulsoft.Report;
using Stimulsoft.Report.Export;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TestIssue.DataSets;

namespace TestIssue
{
    public class ReportGenerator
    {
        StiReport mainReport = new StiReport();
        ParentChildDataSet ds = new ParentChildDataSet();

        public void GenerateReport()
        {

            // load templates
            var template1 = new T1();
            template1.Dictionary.SynchronizeBusinessObjects();
            template1.Render(false);
            var template2 = new T2();

            FillDataset();
            template2.RegData(ds);
            template2.Dictionary.SynchronizeBusinessObjects();
            template2.Render(false);

            // main
            mainReport.SubReports.Add(template1);
            mainReport.SubReports.Add(template2);
            mainReport.Dictionary.SynchronizeBusinessObjects();
            mainReport.Render(false);

            // output
            var mainReportData = WriteToByteArray(mainReport);
            WriteToPdf("Output.pdf", mainReportData);
        }

        private void FillDataset()
        {
            for (int i = 0; i <= 100; i++)
            {
                var parentRow = ds.Parent1.NewParent1Row();
                parentRow.col0 = i.ToString();
                parentRow.col1 = (100 - i).ToString();
                ds.Parent1.Rows.Add(parentRow);
            }
        }

        private void WriteToPdf(string pdfName, byte[] data)
        {
            var path = Environment.CurrentDirectory;
            var filePath = Path.Combine(path, pdfName);
            using (FileStream file = File.Create(filePath))
            {
                file.Write(data, 0, data.Length);
            }
            Process.Start(filePath);
        }

        private byte[] WriteToByteArray(StiReport stiReport)
        {
            var sw = new Stopwatch();
            sw.Start();
            var settings = new StiPdfExportSettings();
            settings.ImageQuality = 1;
            settings.ImageResolution = 200;
            settings.CreatorString = "JK Test";
            settings.Compressed = true;
            settings.ImageCompressionMethod = StiPdfImageCompressionMethod.Jpeg;
            settings.UserAccessPrivileges = StiUserAccessPrivileges.All;

            using (var stream = new MemoryStream())
            {
                stiReport.ExportDocument(StiExportFormat.Pdf, stream, settings);
                sw.Stop();
                return stream.ToArray();
            }
        }
    }
}
